"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importStar(require("axios"));
// 自定义错误类
class CustomError extends Error {
    constructor(message) {
        super(message);
        this.name = "CustomError";
    }
}
/**
 * AxiosFactory 类用于创建和配置 Axios 实例。
 * 处理登录认证并管理 cookies 和 tokens。
 */
class AxiosFactory {
    /**
     * 构造函数初始化 AxiosFactory。
     * @param config - AxiosFactory 的配置参数。
     * @param axiosConfig - Axios 的配置参数。
     * @param cache - 缓存配置（目前未使用）。
     */
    constructor(config, axiosConfig, cache = {}) {
        this.cookies = new Map();
        this.tokens = {};
        this.verified = false;
        this.maxCookies = 50; // 设置最大 cookie 数量
        this.retryDelays = [1000, 5000, 10000, 30000]; // 设置重试延迟时间（毫秒）
        this.config = config;
        this.axiosClient = this.createAxiosClient(axiosConfig);
    }
    /**
     * 初始化方法，通过调用登录方法进行认证。
     */
    async initialize() {
        await this.login();
    }
    // 登录方法，发送登录请求并处理响应。
    async login(retryAttempt = 0) {
        const loginConfig = {
            url: this.config.auth.uri,
            method: "POST",
            data: this.config.auth.data,
        };
        try {
            await this.axiosClient(loginConfig);
            this.verified = true;
            console.log(`httpApp:${this.config.name}初始化成功`);
        }
        catch (error) {
            if ([502, 503, 504].includes(error?.response?.status)) {
                const delay = this.retryDelays[Math.min(retryAttempt, this.retryDelays.length - 1)];
                console.warn(`登录失败，重试第 ${retryAttempt + 1} 次，等待 ${delay / 1000} 秒...`);
                await new Promise(resolve => setTimeout(resolve, delay));
                return this.login(retryAttempt + 1);
            }
            console.error(`httpApp:${this.config.name}初始化失败`, error);
            throw new CustomError(`Failed to initialize httpApp: ${this.config.name}`);
        }
    }
    /**
     * 创建 Axios 实例并设置拦截器。
     * @param axiosConfig - Axios 的配置参数。
     * @returns 创建的 Axios 实例。
     */
    createAxiosClient(axiosConfig) {
        const config = {
            baseURL: "",
            withCredentials: true,
            timeout: 10000,
            ...axiosConfig,
        };
        const api = axios_1.default.create(config);
        this.setupInterceptors(api);
        return api;
    }
    /**
     * 设置请求和响应拦截器。
     * @param api - Axios 实例。
     */
    setupInterceptors(api) {
        // 请求拦截器
        api.interceptors.request.use((config) => {
            const headers = new axios_1.AxiosHeaders();
            headers.set("cookie", Array.from(this.cookies.values()).join(";"));
            if (this.config.headers) {
                Object.entries(this.config.headers).forEach(([key, value]) => {
                    headers.set(key, value);
                });
            }
            Object.entries(this.tokens).forEach(([key, value]) => {
                headers.set(key, value);
            });
            config.headers = headers;
            return config;
        });
        // 响应拦截器
        api.interceptors.response.use((response) => {
            if (response.headers["set-cookie"]) {
                response.headers["set-cookie"].forEach((cookie) => {
                    const cookieName = cookie.split("=")[0].trim();
                    this.cookies.set(cookieName, cookie);
                });
                // 限制 cookie 数量
                if (this.cookies.size > this.maxCookies) {
                    const cookiesArray = Array.from(this.cookies.entries());
                    this.cookies = new Map(cookiesArray.slice(-this.maxCookies));
                }
            }
            this.config.auth.tokenKeys.forEach((key) => {
                if (key in response.headers) {
                    this.tokens[key] = response.headers[key];
                    console.log(key, this.tokens[key]);
                }
            });
            return response; // 直接返回响应
        }, async (error) => {
            if (error.response?.status === 401) {
                console.warn(`检测到 401 错误，重新登录...`);
                this.verified = false;
                await this.login();
                return api.request(error.config);
            }
            return Promise.reject(error);
        });
    }
}
exports.default = AxiosFactory;
